----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FSM_RX is
	port(
		clk : in std_logic;
		RX : in std_logic;
		RX_RDY : out std_logic; 
		RX_DATA : out std_logic_vector(7 downto 0);
		BAUD_EN : in std_logic;
		RST : in std_logic
	);
end FSM_RX;

architecture Behavioral of FSM_RX is

	signal BIT_CNT : std_logic_vector(2 downto 0);
	signal BAUD_CNT : std_logic_vector(3 downto 0);
	type state_type is (idle, start, bit1, stop, wait1);
	signal state : state_type;
	signal bit_state : std_logic;
	signal RX_aux : std_logic_vector(7 downto 0);
begin
	
	process1: process (clk, RST, BAUD_EN)
	begin
		if (RST ='1') then
			state <=idle;
		elsif (clk' event and clk = '1' ) then
			if BAUD_EN = '1' then
				case state is
					when idle => if RX='0' then
													state <= start;
													BIT_CNT <= "000";
													BAUD_CNT <= "0000";
												else
													state <= idle;
													BIT_CNT <= "000";
													BAUD_CNT <= "0000";
											end if;
				when start => 
													if BAUD_CNT = "0111" then
														if RX = '0' then
															state <= bit1;
															BIT_CNT <= "000";
															BAUD_CNT <= "0000";
														else
															state <= idle;
															BIT_CNT <= "000";
															BAUD_CNT <= "0000";
														end if;
													else
														state <= start;
														BIT_CNT <= "000";
														BAUD_CNT <= BAUD_CNT + 1;
													end if;
												
				when bit1 => if BAUD_CNT = "1111" then
												if BIT_CNT = "111" then
													state <= stop;
													BIT_CNT <= "000";
													BAUD_CNT <= "0000";
												else
													state <= bit1;
													BAUD_CNT <= "0000";
													BIT_CNT <= BIT_CNT + 1;
												end if;
											else
												state <= bit1;
												BAUD_CNT <= BAUD_CNT + 1;
											end if;
												
				when stop =>	if BAUD_CNT = "1111" then
													state <= wait1;
													BIT_CNT <= "000";
													BAUD_CNT <= "0000";
												else
													state <= stop;
													BIT_CNT <= "000";
													BAUD_CNT <= BAUD_CNT + 1;
												end if;
											
				when wait1 => if BAUD_CNT = "0111" then
													state <= idle;
													BIT_CNT <= "000";
													BAUD_CNT <= "0000";

												else
													state <= wait1;
													BAUD_CNT <= BAUD_CNT + 1;
													BIT_CNT <= "000";
												end if;
				end case;
				end if;
			end if;
		end process process1;
		
		
		process2: process(state, BIT_CNT)
		begin
			case state is	
				when idle => RX_RDY <= '0';     bit_state <= '0';
				when start => RX_RDY <= '0';    bit_state <= '0';
				when bit1 => RX_RDY <= '0';	    bit_state <= '1';
				when stop => RX_RDY <= '0';	    bit_state <= '0';
				when wait1 => RX_RDY <= '1';	bit_state <= '0';
			end case;
		end process process2;
		
		process_RX: process(clk)
		begin
			if clk = '1' and clk' event then 
				if BAUD_EN = '1' then 
					if bit_state  = '1' then
						if BAUD_CNT = "1111" then
							RX_aux(conv_integer(BIT_CNT)) <= RX;
						end if;
					end if;
				end if;
			end if;
		end process process_RX;
		
		RX_DATA <= RX_aux;

end Behavioral;