----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity Serial_Com is
				port (
				clk : in std_logic;
				sw : in std_logic_vector(15 downto 0);
				btn : in std_logic_vector(4 downto 0);
				led : out  STD_LOGIC_VECTOR (15 downto 0);
				an : out STD_LOGIC_VECTOR (3 downto 0);
                cat : out STD_LOGIC_VECTOR (6 downto 0);
				RX : in std_logic;
                TX : out std_logic;
             analog: in STD_LOGIC; -- analog output -- ignored 
             rx_sonar: out STD_LOGIC; -- set to 1
             tx_sonar: in STD_LOGIC; -- ignored
             pw: inout STD_LOGIC -- the PWM distance signal		
                );
end Serial_Com;

architecture Behavioral of Serial_Com is
--====== TX FSM =============
component tx_fsm is
    Port ( clk : in STD_LOGIC;
           baud_en : in STD_LOGIC;
           rst : in STD_LOGIC;
           tx_en : in STD_LOGIC;
           tx_data : in STD_LOGIC_VECTOR (7 downto 0);
           tx : out STD_LOGIC;
           tx_rdy : out STD_LOGIC);
end component;
--=============================
--==== DEBOUNCER ===============
component mpg is
    Port ( btn : in  STD_LOGIC_vector(4 downto 0);
	 clk: in std_logic;
	 en: out std_logic_vector(4 downto 0));
end component;
--===========================

--==== SONAR ======================
component sonar is
  Port (
  clk: in STD_LOGIC;
  analog: in STD_LOGIC; -- analog output -- ignored 
  rx: out STD_LOGIC; -- set to 1
  tx: in STD_LOGIC; -- ignored
  pw: inout STD_LOGIC; -- the PWM distance signal
  range_i: out STD_LOGIC_VECTOR(7 downto 0) -- output is the range in inches
   );
end component;
--======================================
--==== 7 segments diplay ===============
 component display is
      Port ( clk : in STD_LOGIC;
          disp : in STD_LOGIC_VECTOR (15 downto 0);
          an : out STD_LOGIC_VECTOR (3 downto 0);
          cat : out STD_LOGIC_VECTOR (6 downto 0));
end component;
    
--=============================

--====== RX FSM =============
component FSM_RX is
	port(
		clk : in std_logic;
		RX : in std_logic;
		RX_RDY : out std_logic; 
		RX_DATA : out std_logic_vector(7 downto 0);
		BAUD_EN : in std_logic;
		RST : in std_logic
	);
end component;
--====================

--==== SIGNALS ===============
	signal baudRate : std_logic_vector(15 downto 0);
	signal baud_enable : std_logic;
	signal ene: std_logic := '0';
	signal enable : std_logic_vector(4 downto 0);
	signal ready,ready2 : std_logic;
	signal start,start2 : std_logic := '0';
	signal oversampling : std_logic_vector(10 downto 0) := (others => '0');
	signal overs2: integer := 0;
	signal RX_DATA_aux  : std_logic_vector(15 downto 0) := (others => '0');
	signal TX_DATA_aux: std_logic_vector(7 downto 0):= (others => '0');
	signal range_sonar_aux: std_logic_vector(7 downto 0):= (others => '0');
	signal disp_data: std_logic_vector(15 downto 0);
	signal tx_en : std_logic;
	signal wait_for_loop_retransmission_tx : integer := 0;
--==========================

begin
	
sonar_max    : sonar port map (
  clk=> clk,
  analog=> analog,
  rx=> rx_sonar,
  tx=> tx_sonar,
  pw=>pw,
  range_i=> range_sonar_aux
   );
seg7_display : display port map(
            clk => clk,
            disp => disp_data,
            an =>an,
            cat =>cat
            );

debouncer_map : mpg port map(
							btn => btn,
							clk => clk,
							en => enable
							);	
												
FSM_map_rx : FSM_RX port map(
						clk => clk,
						RX => RX,
						RX_RDY => ready,
						RX_DATA => RX_DATA_aux(7 downto 0),
						BAUD_EN => start,
						RST => enable(1)
	                   );
													
FSM_map_tx : tx_FSM port map(
                        clk,
                        start2,
                        enable(1),
                        tx_en,
                        tx_data_aux,
                        TX,
                        ready2
                        );
			
    --visualization mux for 7 seg display
    process(sw(15 downto 13))
    begin
        case sw(15 downto 13) is
            when "000"  => disp_data <= x"0000";
            when "001"  => disp_data <= x"00" & RX_DATA_aux(7 downto 0);
            when "011"  => disp_data <= x"00" & range_sonar_aux;
            when "010"  => disp_data <= x"0000";
            when "100"  => disp_data <= x"0000";
            when "101"  => disp_data <= x"0000";
            when others => disp_data <= sw;
        end case;
    end process;
    
    
	process(clk, enable, start2)
	begin
	   if rising_edge(clk) then
           --if enable(0) = '1' then
           if wait_for_loop_retransmission_tx = 27774 then
               tx_en <= '1';
            elsif start2 = '1' then
               tx_en <= '0';
           end if;
       end if;
	end process;
	
	process(clk,wait_for_loop_retransmission_tx) --counter to transmit the sonar info continuously each 16 bits based on transmission sampling
        begin
           if rising_edge(clk) then
               if wait_for_loop_retransmission_tx = 27774 then
                 wait_for_loop_retransmission_tx <= 0;
             else
                 wait_for_loop_retransmission_tx <=wait_for_loop_retransmission_tx + 1;
             end if;
           end if;
    end process;
	--tx_data_aux <= sw(7 downto 0);	
	tx_data_aux <= range_sonar_aux;									
    led(7 downto 0) <= RX_DATA_aux(7 downto 0);
	
    oversampling_p : process(clk, oversampling)
	begin
		if clk' event  and clk = '1' then
		-- 867 = (round(100MHz / 115200)) - 1
			if oversampling = 53 then
				start <= '1';
				oversampling <= (others => '0');
			else
				oversampling <= oversampling + 1;
				start <= '0';
			end if;
		end if;
	end process;
	
	oversampling_p2 : process(clk, overs2)
        begin
            if clk' event  and clk = '1' then
            -- 867 = (round(100MHz / 115200)) - 1
                if overs2 = 867 then
                    start2 <= '1';
                    overs2 <= 0;
                else
                    overs2 <= overs2 + 1;
                    start2 <= '0';
                end if;
            end if;
        end process;
	
end Behavioral;